rm(list=ls())
options(digits=5)

setwd(dirname(rstudioapi::getActiveDocumentContext()$path))
getwd()

## The tips below are useful in shaping up student laptop ratings csv files:
## Regexp and replace on the console:
## find . -name \*.csv -exec sed -i "s/;/,/g" {} \; ## replace every ";" with ","
## find . -name \*.csv -exec sed -i "s/\"//g" {} \; ## remove all """ (double quotation)
## find . -name \*.csv -exec sed -i "s/,\+/,/g" {} \; ## remove multiple occurrences of comma (,)
## find . -name \*.csv -exec sed -i "s/,$//" {} \; ## remove comma (,) at the end of each line
##
## find . -name \*.csv -exec sed -i "s/,//g" {} \; ## remove comma (,) at the end of each line: NO! This does something else. But what?

##############################################################################################
## design of a choice experiment 
##############################################################################################

library(support.CEs)  # package for survey construction 
library(magrittr)
library(tidyverse) # include AFTER support.CEs because support.CEs require MASS package, which has select command, which interferes with tidyverse/dplyr select command
library(stringr)
library(gridExtra)
library(writexl)


## generate a balanced set of product profiles for survey
provider.survey <-
  Lma.design(attribute.names = 
               # list(Brand = c("HP", "Toshiba", "Casper", "Sony"),
               #      Processor = c("Intel Core i5", "Intel Core i7"),
               #      Speed = c("1.8 GHz", "2.2 GHz", "2.4 GHz", "2.6 GHz"), ## GHz
               #      Memory = c("4 GB","8 GB"), ## GB
               #      HardDisk = c("500 GB", "750 GB"),
               #      Screen = c("15.6 in", "17.3 in"),
               #      Price = c("1500 TL", "1700 TL", "2000 TL", "3500 TL")), ## TL), ## inches
             list(Brand = c("HP", "Toshiba", "Casper", "Sony"),
                  Processor = c("Intel Core i5", "Intel Core i7"),
                  Speed = c("1.8 GHz", "2.2 GHz", "2.4 GHz", "2.6 GHz"), ## GHz
                  Memory = c("8 GB","16 GB"), ## GB
                  HardDisk = c("750 GB", "1 TB"),
                  Screen = c("15.6 in", "17.3 in"),
                  Price = c("6.000 TL", "7.000 TL", "8.500 TL", "10.000 TL")), ## TL), ## inches
             nalternatives = 1, nblocks=1, row.renames = FALSE, seed=9999)
print(questionnaire(provider.survey))  # print survey design for review

sink("questions_for_survey.txt")  # send survey to external text file
questionnaire(provider.survey)
sink() # send output back to the screen

## extract survey form which is more suitable to write in a
## comma-separated file
(survey = provider.survey$alternatives$alt.1[,c(2,4:10)]) 
colnames(survey)[1] ="Alternative"

## add an empty rank columns for survey participant
(survey = data.frame(survey, RANK=rep("",nrow(survey)))) 

## write the survey to a comma-separated file after the instructions
## on how the survey has to be filled.
# msg = sprintf("## Please rank %d laptops below\n## each with exactly one number\n## between 1 and %d\n## in the RANK column (last column):\n##  1 = most preferred,\n## %d = least preferred.\n## You may sort the rows\n## with respect to columns\n## if it helps.\n", nrow(survey), nrow(survey), nrow(survey))
# cat(msg, file = "laptops.csv")
#write_csv(survey, path="laptops.csv", append = TRUE, col_names = TRUE)
survey$Instructions <-  rep("", nrow(survey))
msg = c(sprintf("Please rank %d laptops on the left each with exactly one number", nrow(survey)),
        sprintf("between 1 and %1$d in the RANK column (last column):", nrow(survey)),
        sprintf("1 = most preferred, %1$d = least preferred.", nrow(survey)),
        "You may sort the rows with respect to columns if it helps.")
survey$Instructions[seq(length(msg))] <- msg

write_xlsx(survey, path="laptops.xlsx")

##############################################################################################
##############################################################################################
#### conjoint analysis of survey results
##############################################################################################
##############################################################################################
detach("package:support.CEs") ## support.CEs needs MASS
detach("package:MASS") ## need to detach MASS because its "select" function interferes with that of tidyverse

response <- read_csv("laptops-hande.csv", comment="#") %>% 
  ## Remove Alternative column -- irrelevant to the conjoint analysis
  select(-Alternative) %>% 
  ## store current the setting in order to get back to it after
  ## finishing the analysis
  mutate(RANK = -RANK) %>% 
  mutate_if(is.character, factor)
response

## switch to sum contrasts before multip regression is fit
options(contrasts=c('contr.sum','contr.poly'))
contr.treatment(levels(response$Brand))

options('contrasts') ## double check
contr.sum(levels(response$Brand))

(res = lm(RANK ~ ., response)) ## calculate the part worths 
(coefs = coef(res)) ## printed in scientific format
round(coefs, digits=8) ## round to 8 digits for more readable format

#####################################################################################
## Calculate part worths of product feature levels
#####################################################################################

## arrange the coefs of linear regression into the form of a data frame
PartWorths0 <- coefs %>% 
  data.frame(partworth=.) %>% 
  rownames_to_column("category") %>% 
  filter(category!="(Intercept)") %>% 
  mutate(levelno=parse_number(category),
         category=str_extract(category, "\\D*")) %>% 
  select(category, levelno, everything())

## find partworths of remaining levels of product features
addtnal <- PartWorths0 %>% 
  group_by(category) %>% 
  summarize(levelno=as.integer(max(levelno)+1), partworth=-sum(partworth))

## match level no's and level names of product features
featureLevels <- response %>% 
  select_if(is.factor) %>% 
  map(levels) %>% 
  unlist() %>% 
  data.frame(level=.) %>% 
  rownames_to_column("category") %>% 
  mutate(levelno=parse_number(category),
         category=str_extract(category, "\\D*")) %>% 
  select(category, levelno, everything())

## finish calculation of partworths of product feature levels
PartWorths <- PartWorths0 %>% bind_rows(addtnal) %>% 
  as_tibble() %>% 
  arrange(category, levelno) %>% 
  left_join(featureLevels, by=c("category", "levelno")) %>% 
  select(-levelno) %>% 
  select(category, level, everything())

#####################################################################################
## Calculate and plot part importances of product features
#####################################################################################

PartImportance <- PartWorths %>% 
  group_by(category) %>% 
  summarize(range=diff(range(partworth))) %>% 
  mutate(importance=range/sum(range)) %>% 
  select(-range) %>% 
  arrange(desc(importance))

## Plot part importance scores

p <- PartImportance %>% 
  ggplot(aes(reorder(category, importance), importance)) +
  geom_bar(stat="identity", aes(fill=category)) +
  labs(x="category") +
  theme(legend.position = "none") +
  coord_flip() +
  labs(x="Product features", y="Part importance")

ggsave(filename="partImportance.pdf", plot=p, dev="pdf")

## Plot part worths

p <- PartWorths %>% 
  left_join(PartImportance, by="category") %>% 
  mutate(category=reorder(category, importance)) %>% 
  ggplot(aes(x=level)) +
  geom_linerange(ymin=0, aes(ymax=partworth, col=category)) +
  geom_point(aes(y=partworth, col=category)) +
  geom_hline(yintercept = 0, col="gray") +
  facet_grid(category ~ ., space="free", scales="free", as.table = FALSE) +
  coord_flip() + 
  theme(legend.position = "none", strip.text.y=element_text(angle=0)) +
  labs(x=NULL, y="Part worths")

ggsave(filename="partWorths.pdf", plot=p, dev="pdf")

#####################################################################################
## Calculate the utilities for the laptops from hepsiburada
#####################################################################################

dd <-  read_csv("laptops-hepsiburada-hande.csv", comment="#") %>% 
  {cbind(., RANK.pred = as.integer(rank(-predict(res, newdata=.))))} %>% 
  as_tibble() %>% 
  select(RANK, RANK.pred, everything())

p <- dd %>% 
  ggplot(aes(RANK.pred, RANK)) +
  geom_abline(intercept=0, slope=1, col="gray") +
  geom_point() +
  geom_line(lty="dashed") +
  labs(x="Predicted RANK", y="Observed RANK") +
  scale_x_continuous(labels = seq(nrow(dd)), breaks = seq(nrow(dd))) +
  scale_y_continuous(labels = seq(nrow(dd)), breaks = seq(nrow(dd)))

ggsave(filename="predictedRANKS.pdf", plot=p, dev="pdf")

#####################################################################################
#####################################################################################
## ## Student surveys for market simulation
#####################################################################################
#####################################################################################


all.output.dir <- "conjoint"

dirname = "student laptop ratings"
outfile.prefix = "all"


## dirname = "./student laptop ratings/Fall 2018"
## outfile.prefix = "F18"

output.dir <- file.path(all.output.dir, outfile.prefix)
dir.create(output.dir, recursive=TRUE, showWarnings = FALSE)

survey.file.list <- list.files(path=dirname, recursive=TRUE, pattern="\\d+\\.csv") %>% 
  file.path(dirname, .)
  
## read survey data
surveyPB <- txtProgressBar(min = 0, max = length(survey.file.list), style=3)
survey.df <- NULL
for (i in seq(survey.file.list)){
  ## extract student ID from file name
  StuID <- str_extract(survey.file.list[i], "\\d{8,8}")
  
  ## read survey data
  response.train <- read_csv(survey.file.list[i], comment="#", col_types = cols()) %>% 
    arrange(Alternative) %>% 
    mutate_if(is.character, factor)
  
  ## build test data name for the same student and read test data
  test.filename <- str_replace(survey.file.list[i], StuID, paste(StuID, "hepsiburada", sep="-"))
  response.test <- read_csv(test.filename, comment="#", col_types = cols()) %>% 
    arrange(Alternative) %>% 
    mutate_if(is.character, factor)
  
  ## add student's ID, train set and test set to survey data frame
  survey.df <- bind_rows(survey.df,
                         data_frame(StuID=StuID, train=list(response.train), test=list(response.test)))
  
  setTxtProgressBar(surveyPB, i)
}
close(surveyPB)

## Check out the train data of student whose StuID ending with 807:
survey.df %>% 
  filter(str_detect(StuID, "807$")) %>% 
  extract2("train")

## fit regression model to -RANKS
analysis.df <- survey.df %>% 
  mutate(model = map(train, ~lm(-RANK ~ ., select(.x, -Alternative))))

# analysis.df$model[[1]]
# analysis.df$preds[[1]]

#####################################################################################
## Calculate and plot part worths of product feature levels for the survey participants
#####################################################################################

## Turn the previous individual part worth calculation into a function
calcPartWorth <- function(response, model){
  ## takes train set and a model fit
  coefs = coef(model)
  
  PartWorths0 <- coefs %>% 
    data.frame(partworth=.) %>% 
    rownames_to_column("category") %>% 
    filter(category!="(Intercept)") %>% 
    mutate(levelno=parse_number(category),
           category=str_extract(category, "\\D*")) %>% 
    select(category, levelno, everything())
  
  ## find partworths of remaining levels of product features
  addtnal <- PartWorths0 %>% 
    group_by(category) %>% 
    summarize(levelno=as.integer(max(levelno)+1), partworth=-sum(partworth))
  
  ## match level no's and level names of product features
  featureLevels <- response %>% 
    select_if(is.factor) %>% 
    map(levels) %>% 
    unlist() %>% 
    data.frame(level=.) %>% 
    rownames_to_column("category") %>% 
    mutate(levelno=parse_number(category),
           category=str_extract(category, "\\D*")) %>% 
    select(category, levelno, everything())
  
  ## finish calculation of partworths of product feature levels
  PartWorths <- PartWorths0 %>% bind_rows(addtnal) %>% 
    as_tibble() %>% 
    arrange(category, levelno) %>% 
    left_join(featureLevels, by=c("category", "levelno")) %>% 
    select(-levelno) %>% 
    select(category, level, everything())
  
  return(PartWorths)
}

## calculate part importance
calcPartImportance <- function(PartWorths){
  PartImportance <- PartWorths %>% 
    group_by(category) %>% 
    summarize(range=diff(range(partworth))) %>% 
    mutate(importance=range/sum(range)) %>% 
    select(-range) %>% 
    arrange(desc(importance))
  return(PartImportance)
}

## calculate predictions on new data 
calcPredictions = function(model, newdata){
  newdata %>% 
    {cbind(., RANK.pred = as.integer(rank(-predict(model, newdata=.), 
                                          ties.method = "random")))} %>% 
  as_tibble()
}


## Plot part worth
plotPartWorth <- function(PartWorths, PartImportance, title=NULL){
  p <- PartWorths %>% 
    left_join(PartImportance, by="category") %>% 
    mutate(category=factor(category), ## use category for consistent coloring
           category2=reorder(category, importance)) %>% ## use category2 for ordering facets
    ##mutate(category=reorder(category, importance)) %>% 
    ggplot(aes(x=level)) +
    geom_linerange(ymin=0, aes(ymax=partworth, col=category)) +
    geom_point(aes(y=partworth, col=category)) +
    geom_hline(yintercept = 0, col="gray") +
    facet_grid(category2 ~ ., space="free", 
               scales="free", as.table = FALSE) +
    coord_flip() + 
    theme(legend.position = "none", strip.text.y=element_text(angle=0)) +
    labs(x=NULL, y="Part worths", title=title)
  
  return(p)
}

# Plot part importance  
plotPartImportance <- function(PartImportance, title=NULL){
  p <- PartImportance %>% 
    ## ggplot(aes(reorder(category, importance), importance)) +
    arrange(category) %>% 
    mutate(category=factor(category)) %>% 
    ggplot(aes(category, importance)) +
    geom_bar(stat="identity", aes(fill=category)) +
    labs(x="category") +
    theme(legend.position = "none") +
    coord_flip() +
    labs(x="Product features", y="Part importance",
         title=title)
  return(p)  
}


## Calculate the part worth and part importance for every respondent
analysis2.df<- analysis.df %>% 
  mutate(partWorth = map2(train, model, calcPartWorth),
         partImportance = map(partWorth, calcPartImportance),
         preds = map2(model, test, calcPredictions),
         ##partWorthPlot= map2(partWorth, partImportance, plotPartWorth),
         ##partImportancePlot = map(partImportance, plotPartImportance))
         censoredStuID = paste0(str_extract(StuID,"^\\d{3,3}"), "...",
                               str_extract(StuID,"\\d{2,2}$")),
         partImportancePlot = map2(partImportance, censoredStuID, plotPartImportance),
         partWorthPlot= pmap(list(partWorth, partImportance, censoredStuID), plotPartWorth))

# analysis2.df$partWorth[[140]]
# analysis2.df$partImportance[[140]]
# analysis2.df$preds[[140]]
# analysis2.df$partWorthPlot[[140]]
# analysis2.df$partImportancePlot[[4]]

## plot part importance of the participants
if (!file.exists(file.path(output.dir, sprintf("partImportanceSurvey2-%s.pdf", outfile.prefix)))){
  pdf(file.path(output.dir, sprintf("partImportanceSurvey2-%s.pdf", outfile.prefix)),
    paper="a4", width=0, height=0)
## grid.arrange(grob=analysis2.df$partImportancePlot, nrow=2, ncol=3)
  marrangeGrob(grob=analysis2.df$partImportancePlot, nrow=3, ncol=2)
  dev.off()
}
## plot partworths of the participants
if (!file.exists(file.path(output.dir, sprintf("partWorthsSurvey2-%s.pdf", outfile.prefix)))){
  pdf(file.path(output.dir, sprintf("partWorthsSurvey2-%s.pdf", outfile.prefix)),
    paper="a4", width=0, height=0)
  marrangeGrob(grob=analysis2.df$partWorthPlot, nrow=3, ncol=2)
  dev.off()
}
#####################################################################################
## Evaluate the conjoint model
#####################################################################################

analysis2.df %>% 
  transmute(first = map_lgl(preds, ~{which.min(.x$RANK)==which.min(.x$RANK.pred)})) %>% 
  unlist %>% unname %>% table() %>% prop.table()

## histogram of number of agreements between observed and predicted ranks
analysis2.df %>% 
  transmute(first = map_int(preds, ~{sum(.x$RANK==.x$RANK.pred)})) %>% 
  unlist %>% 
  unname() %>% 
  data.frame(agree=.) %>% 
  ggplot(aes(agree)) +
  geom_bar()

## histogram of average mean absolute deviation between observed and predicted ranks
analysis2.df %>% 
  transmute(first = map_dbl(preds, ~{mean(abs(.x$RANK-.x$RANK.pred))})) %>% 
  unlist %>% 
  unname() %>% 
  data.frame(diff=.) %>% 
  ggplot(aes(diff)) +
  geom_histogram(binwidth = .35)

#####################################################################################
## Predict the market shares of laptops, brands, prices, screen sizes,...
#####################################################################################

## Predicted market shares of laptops on sale at hepsiburada.com. We assume that laptops never go out of stock, and participants always get their first choices.

market.shares <- analysis2.df %>% 
  transmute(pred.first.choice = map_int(preds, ~.x$Alternative[which(.x$RANK.pred==1)]),
            obs.first.choice = map_int(preds, ~.x$Alternative[which(.x$RANK==1)])) %>% 
  gather(first.choice, Alternative) %>% 
  mutate(first.choice=str_extract(first.choice, ".*?(?=\\.)")) %>% 
  group_by(first.choice) %>% 
  count(Alternative) %>% 
  mutate(market.share=n/sum(n)) %>% 
  left_join(response.test, by=c("Alternative"))

## Market shares of laptops
market.shares %>% 
  select(-n) %>% 
  spread(first.choice, market.share)

market.shares %>% 
  mutate(Alternative=factor(Alternative)) %>% 
  ggplot(aes(Alternative, market.share)) +
  geom_bar(stat="identity", aes(fill=Alternative)) +
  scale_y_continuous(labels=scales::percent) +
  facet_grid(first.choice~.) +
  theme(legend.position = "none") +
  labs(y="Market share")

# Market shares of brands
market.shares %>% 
  group_by(first.choice, Brand) %>% 
  summarize(market.share=sum(market.share)) %>% 
  spread(first.choice, market.share) 
  
market.shares %>% 
  group_by(first.choice, Brand) %>% 
  summarize(market.share=sum(market.share)) %>% 
  ggplot(aes(Brand, market.share)) +
  geom_bar(stat="identity", aes(fill=Brand)) +
  scale_y_continuous(labels = scales::percent) +
  facet_grid(first.choice~.) +
  theme(legend.position = "none") +
  labs(y="Market share")

p <- market.shares %>% 
  group_by(first.choice, Brand) %>% 
  summarize(market.share=sum(market.share)) %>% 
  ungroup() %>% 
  mutate(first.choice = case_when(
           first.choice=="obs" ~ "observed",
           TRUE ~ "simulated"
         )) %>% 
  ggplot(aes(Brand, market.share)) +
  geom_col(aes(fill=first.choice), position = "dodge") +
  geom_text(aes(label=scales::percent(market.share),
                y=market.share-0.01,
                group=first.choice),
            position=position_dodge(.9))+
  scale_y_continuous(labels = scales::percent) +
  labs(y="Market share", fill=NULL) +
  theme(legend.position = "top")
p
ggsave(filename=sprintf("%s/marketshare-%s.pdf", output.dir, outfile.prefix), 
       plot=p, device="pdf")


p <- market.shares %>% 
  ungroup() %>% 
  mutate(Brand=paste(Brand, Alternative,sep="."),
    first.choice = case_when(
           first.choice=="obs" ~ "observed",
           TRUE ~ "simulated"
         )) %>% 
  ggplot(aes(Brand, market.share)) +
  geom_col(aes(fill=first.choice), position = "dodge") +
  geom_text(aes(label=scales::percent(market.share),
                y=market.share-0.01,
                group=first.choice),
            position=position_dodge(.9))+
  scale_y_continuous(labels = scales::percent) +
  labs(y="Market share", fill=NULL) +
  theme(legend.position = "top")
p
ggsave(filename=sprintf("%s/marketshares-ForEachModel-%s.pdf", 
                        output.dir, outfile.prefix), 
       plot=p, device="pdf")

# Market shares of prices
market.shares %>% 
  group_by(first.choice, Price) %>% 
  summarize(market.share=sum(market.share)) %>% 
  spread(first.choice, market.share) 

market.shares %>% 
  group_by(first.choice, Price) %>% 
  summarize(market.share=sum(market.share)) %>% 
  ggplot(aes(Price, market.share)) +
  geom_bar(stat="identity", aes(fill=Price)) +
  scale_y_continuous(labels = scales::percent) +
  facet_grid(first.choice~.) +
  theme(legend.position = "none") +
  labs(y="Market share")

# Market shares of screen sizes
market.shares %>% 
  group_by(first.choice, Screen) %>% 
  summarize(market.share=sum(market.share)) %>% 
  spread(first.choice, market.share) 

market.shares %>% 
  group_by(first.choice, Screen) %>% 
  summarize(market.share=sum(market.share)) %>% 
  ggplot(aes(Screen, market.share)) +
  geom_bar(stat="identity", aes(fill=Screen)) +
  scale_y_continuous(labels = scales::percent) +
  facet_grid(first.choice~.) +
  theme(legend.position = "none") +
  labs(y="Market share")

#####################################################################################
## Predict substitution rate of Casper laptops
#####################################################################################

# What is Casper’s substitution rate for each of the other laptop brands in Table 1? (If, for example, Sony is unavailable, what is the percentage of customers who choose Casper laptop?)

substitutes0 <- analysis2.df %>% 
  transmute(
      substitute.obs = map(preds, ~{.x %>% arrange(RANK) %>% 
          select(Alternative) %>% 
          mutate(Substituted_By= lag(Alternative)) %>% 
          rename(Alternative.obs=Alternative,
                 Substituted_By.obs=Substituted_By)}),
      substitute.pred = map(preds, ~{.x %>% arrange(RANK.pred) %>% 
          select(Alternative) %>% 
          mutate(Substituted_By= lag(Alternative)) %>% 
          rename(Alternative.pred=Alternative,
                 Substituted_By.pred=Substituted_By)})
      ) %>% 
  unnest(substitute.obs, substitute.pred) %>% 
  mutate(id=seq(n())) %>% 
  gather(var, val, -id) %>% 
  separate(var, into=c("var","type"), sep="\\.") %>% 
  spread(var, val) %>% 
  na.omit() %>% 
  count(type, Alternative, Substituted_By, sort=TRUE)
substitutes0

substitutes <- substitutes0 %>% 
  count(type, Alternative, wt=n) %>% 
  right_join(substitutes0, by=c("type", "Alternative")) %>% 
  mutate(prop=n/nn)

## number of substitutes  
substitutes %>% 
  select(type, Alternative, Substituted_By, n) %>% 
  spread(Substituted_By, n) 

## proportion of substitutes
substitutes %>% 
  select(type, Alternative, Substituted_By, prop) %>% 
  spread(Substituted_By, prop)

p <- substitutes %>%
  mutate(type=case_when(
    type=="obs" ~ "Observed",
    TRUE ~ "Simulated"
  )) %>% 
  rename(Frequency=prop) %>% 
  ggplot(aes(x=Substituted_By, y=-Alternative, fill=Frequency))  +
  geom_tile() +
  geom_text(aes(label=scales::percent(round(Frequency,2))), size=3) +
  ##scale_fill_distiller(palette="YlOrRd")+
  scale_x_continuous(breaks=1:16, labels=1:16, position="top")+
  scale_y_continuous(breaks=-(1:16), labels=1:16)+
  scale_fill_distiller(palette="RdYlBu", labels=scales::percent)+
  coord_equal() +
  labs(y="Alternative", x="Substituted by") +
  facet_grid(.~type) +
  theme(legend.position = "bottom")
p
# ggsave(filename=sprintf("%s/substitution2-%s.pdf", output.dir, outfile.prefix),
#        plot=p,
#        device="pdf", height=4, width=6)

## reorder the rows and columns in the decreasing order of market share

## order laptops in test set according to their market shares
response.test %>% 
  select(-RANK) %>% 
  left_join(filter(market.shares, first.choice=="pred"), by="Alternative") %>% 
  arrange(desc(market.share))
  
# substitution matrix with to rows and columns ordered according to the simulated market shares
p <- substitutes %>%
  mutate(Alternative=
           factor(Alternative, 
                  levels=rev(arrange(filter(market.shares, first.choice=="pred"), 
                                          desc(market.share))$Alternative)),
         Substituted_By=
           factor(Substituted_By, 
                  levels=arrange(filter(market.shares, first.choice=="pred"), 
                                 desc(market.share))$Alternative),
         type=case_when(
                   type=="obs" ~ "Observed",
                   TRUE ~ "Simulated"
                )) %>% 
  rename(Frequency=prop) %>% 
  ggplot(aes(x=Substituted_By, y=Alternative, fill=Frequency))  +
  geom_tile() +
  geom_text(aes(label=scales::percent(round(Frequency,2))), size=3) +
  scale_fill_distiller(palette="RdYlBu", labels=scales::percent)+
  coord_equal() +
  labs(y="Alternative", x="Substituted by") +
  facet_grid(.~type) +
  theme(legend.position = "bottom")
p
ggsave(filename=sprintf("%s/substitution2-%s.pdf", output.dir, outfile.prefix),
       plot=p,
       device="pdf", height=4, width=6)


market.shares2 <- market.shares %>% 
  ungroup() %>% 
  filter(first.choice=="pred") %>% 
  select(Alternative, Brand, market.share) %>% 
  mutate(Brand=paste(Brand, Alternative, sep=".")) %>% 
  arrange(desc(market.share))
market.shares2

## Substitute models
p <- substitutes %>% 
  select(-nn, -n) %>% 
  left_join(select(response.test, Brand, Alternative), by="Alternative") %>% 
  mutate(Alternative=paste(Brand, Alternative, sep=".")) %>% 
  select(-Brand) %>% 
  left_join(select(response.test, Brand, Alternative), 
             by=c("Substituted_By"="Alternative")) %>% 
  mutate(Substituted_By=paste(Brand, Substituted_By, sep=".")) %>% 
  select(-Brand) %>% 
  mutate(type=case_when(
    type=="obs" ~ "observed",
    TRUE~"simulated"
  ),
  Alternative=factor(Alternative, levels=market.shares2$Brand),
  Substituted_By=factor(Substituted_By, levels=rev(market.shares2$Brand))) %>% 
  ggplot(aes(Substituted_By, prop)) +
  geom_col(aes(fill=type), position="dodge") +
  scale_y_continuous(labels=scales::percent) +
  facet_wrap(~Alternative, ncol = 2, scales="free_x") +
  coord_flip() +
  theme(legend.position = "top") +
  labs(x="Substituted by", y=NULL, fill=NULL)
p
ggsave(filename=sprintf("%s/substitution-%s.pdf", output.dir, outfile.prefix),
       plot=p, device="pdf")

#####################################################################################
## 1(c) iii What are the alternative products market shares?
#####################################################################################

analysis2.df$partWorth[[1]] ## the first respondent's partworths
analysis2.df$model[[1]] ## the first respondent's regression model

market.shares ## current market shares

## Current market with product features compactified
market.cur <- response.test %>% 
  ## convert factors to strings
  mutate_if(is.factor, as.character) %>% 
  select(-RANK) %>% 
  nest(-Alternative, .key="Features")

market.cur

## Generate all possible new product designs for Casper to be considered (below each new market 
## has only one new product, but does not have to be.)
## Each new product nust have the alternative that it replaces in the current market.
new.product.df <- with(unnest(market.cur),
  expand.grid(Alternative=4L, Brand="Casper",
              Processor=unique(Processor),
              Speed=unique(Speed),
              Memory=unique(Memory),
              HardDisk=unique(HardDisk),
              Screen=unique(Screen),
              Price=unique(Price),
              ## keep strings as strings
              ## otherwise new products with the same values but different factor levels are
              ## considered different!
              stringsAsFactors = FALSE) 
) %>% 
  as_tibble() %>% 
  mutate(Market=seq(n())) %>% 
  nest(-Market,-Alternative, .key="Features") %>% 
  nest(-Market, .key="NewProducts")

## new.product.df
## new.product.df$NewProducts[[1]] %>% unnest()
## dim(new.product.df)

## Construct new markets
new.markets.df <- new.product.df %>% 
  mutate(AllProducts=map(NewProducts, ~{
    market.cur %>% 
      filter(Alternative!=.x$Alternative) %>% 
      bind_rows(.x) %>% 
      arrange(Alternative)
    }),
    dublicateProducts = map_lgl(AllProducts, ~{
      y <- select(.x,-Alternative);
      ## remove a market where the new product is the same as one of the other existing products or if the new market is the same as the current market.
        {(!identical(y, unique(y))) || identical(y, select(market.cur, -Alternative))}
      })
    ) %>% 
  filter(!dublicateProducts) %>% 
  select(-dublicateProducts)
                                     
## Predictions on the ranks of the alternative products in each market by all market survey participants
time.new.markets.preds <- system.time({
new.markets.df2 <- new.markets.df %>% 
  mutate(Preds = map(AllProducts, 
                     ~{ 
                       y <- unnest(.x) 
                       map2_dfr(analysis2.df$StuID, analysis2.df$model, ~{
                         predict(.y, newdata=y) %>% 
                          multiply_by(-1)  %>% 
                           rank(ties.method="random") %>% 
                           data.frame(StuID=.x, Rank=., stringsAsFactors=FALSE) %>% 
                           rownames_to_column("Alternative") %>% 
                           mutate(Alternative=as.integer(Alternative)) %>% 
                           select(StuID, everything()) %>% 
                           as_tibble()
                         })
                       }))
})

# dim(new.markets.df2)
# new.markets.df2
# new.markets.df2$Preds[[1]]
# new.markets.df2$Preds[[1]] %>% filter(StuID=="21200863")
# new.markets.df$AllProducts[[1]] %>% unnest %>% count(Alternative)
# new.markets.df2$Preds[[1]]

## Calculate the market shares of products and brands
new.markets.df3 <- new.markets.df2 %>% 
  mutate(market.shares.products = map2(Preds, AllProducts,
                                       ~{.x %>% filter(Rank==1) %>% 
                                           select(-Rank) %>% 
                                           count(Alternative) %>% 
                                           mutate(prop = n/sum(n)) %>% 
                                           left_join(unnest(.y), by="Alternative")
                              }),
         market.shares.brands = map(market.shares.products,
                                    ~{.x %>% 
                                        group_by(Brand) %>% 
                                        summarize(n=sum(n), prop=sum(prop))
                                      })
         )

## new.markets.df3$market.shares.products[[1]]
## new.markets.df3$market.shares.brands[[1]]
## new.markets.df3$market.shares.brands[[1]] %>% select(-n) %>% spread(Brand, prop)

new.markets.df4 <- new.markets.df3 %>% 
  mutate(market.shares.brands2 = map(market.shares.brands, 
                                    ~{
                                      .x %>% 
                                        select(-n) %>% 
                                        spread(Brand, prop)
                                    }),
         Price = parse_number(map_chr(NewProducts, ~{unnest(.x)$Price}))
         ) %>% 
  unnest(market.shares.brands2) %>% 
  arrange(desc(Casper), Price)

market.shares.brands <- market.shares %>% 
  filter(first.choice=="pred") %>% 
  group_by(Brand) %>% 
  summarise(market.share=sum(market.share))

product.curr <- market.cur %>% unnest() %>% 
  slice(4) %>% 
  select(-Alternative, -Brand) %>% 
  unlist %>% 
  paste(collapse="|")

p <- new.markets.df4 %>% 
  mutate(text=map_chr(NewProducts, ~{unnest(.x) %>% 
      select(-Alternative,-Brand) %>% 
      unlist %>% 
      paste(collapse="|") %>% 
      paste(product.curr, sep="\n")})) %>% 
  ggplot(aes(Price, Casper, text=text)) +
  geom_jitter(alpha=0.7) +
  geom_hline(yintercept = 
               market.shares.brands[market.shares.brands$Brand=="Casper","market.share"][[1]]) +
  geom_vline(xintercept = 
               parse_number(filter(unnest(market.cur),Alternative==4)[["Price"]])) +
  labs(y="Market share")

library(plotly)

pp <- ggplotly(p, tooltip = c("y","text") )
pp

save.image(file.path(output.dir, sprintf("conjoint2-%s.RData", outfile.prefix)))
